package holmos.webtest.junitextentions.annotations;

import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

/**
 * 用来指示一个case所用的具体的参数信息:
 * <li>如果是数据库类型,那么如果Source返回结果的id所在行的信息(从0开始)</li>
 * <li>如果是文件类型，在文件中有id这样的字段，并且为整型，那么id代表这些id所在行的信息</li>
 * <li>threadPoolSize:由于一个case的参数有可能会有多组，那么可以采用多线程执行，如果threadPoolSize大于参数组数，那么实际建立的线程池大小为参数组数</li>
 * <li>sourceID:表示一个方法的参数是在SourceId指示的目标文件或者数据库中，如果当前方法有@Source定义，那么这个值无效</li>
 * 
 * @author 吴银龙(sunshine307087558@gmail.com)
 * */
@Retention(RetentionPolicy.RUNTIME)
@Target(ElementType.METHOD)
public @interface Parameter {
	int[] value() default {};
	int threadPoolSize() default 1;
	int sourceID() default -1;
}
